from socket import *
from pickle import *
from TaskQueue import *
from LogLevel import *
from thread import *
from Request import *
from Config import *
from Connection import *
from AllCode import *
from Result import *
from getpass import *
from select import *
from os import *
from Command import *

# Whatever result or request are passed through this
# class, and it gives decision whether the client
# has ended
class ClientConnectionMonitor:

	def __init__(self, log):
		self.log = log
		self.waitForHello = 1
		self.error = 0
		self.waitForResult = 0
		self.fetchResult = 0
		self.waitForAuthReport = 0

	# Updates all the boolean conditions based on
	# request or result
	def updateRules(self, item):
		if isinstance(item, Request):

			# If send a request of event, then first
			# thing is to setup for polling the media.
			# i.e. the authetication process
			if item.isEvent():
				self.waitForAuthReport = 1
			else:
				self.waitForAuthReport = 0

			# If this is FETCH_RESULT command request
			# then keeps waiting to receive incoming result
			# data. Until the parent of the result object
			# is FETCH_RESULT command.
			if item.isCommand():
				if item.getCommandCode() == CommandCode.FETCH_RESULT:
					self.fetchResult = 1
				self.waitForResult = 1
			else:
				self.fetchResult = 0
				self.waitForResult = item.waitForResult()

		elif isinstance(item, Result):

			# Mark down if we have received a hello message from
			# server
			if item.isCommand():
				if item.getCommandCode() == CommandCode.HELLO_MSG:
					self.waitForHello = 0
				else:
					self.waitForResult = 0

			# Mark down if we have received an authetication result
			# and mark error if this fails
			elif item.parentType == RequestCode.AUTH:
				self.waitForAuthReport = 0

				if not item.isSuccess():
					self.error = 1

				# If the parent of this authetication command is
				# from a command, then we don't need to wait for
				# further result. That is it.
				if (isinstance(item.parentObj, Command) and
					item.parentObj.checkArgs(CommandCode.POLL,
											 CommandCode.EMAIL,
											 CommandCode.SETUP_USER)):
					self.waitForResult = 0

				# Otherwise it's generate from an authetication
				# result is generated from an event command
				
			else:
				self.waitForResult = 0

			# If a FETCH_RESULT request has been previously submitted
			# keep waiting until the parent of result obj ==
			# fetch_result command obj
			if self.fetchResult:
				if (item.parentID == self.waitForID and
					item.parentObj.id == self.waitForID):
					self.waitForResult = 0
				else:
					self.waitForResult = 1

	# Based on the current condition, close down the connection ?
	# returns 1 yes, 0 - no
	def isConnFinish(self):

		if self.error:
			return 1

		if self.waitForAuthReport:
			return 0

		if self.waitForResult:
			return 0

		if self.waitForHello:
			return 0

		return 1
		
class Client(Connection):

	server = None
	log    = None

	# Whatever the request submitted to the client, it
	# puts into the queue
	taskQueue = None

	# currentRtnCode is the return code of the most
	# recent task received by the client
	currentRtnCode = None
	
	def __init__(self):
		if GlobalConfig.xmlOutput:
			sys.stdout.write("<?xml version=\"1.0\"?>\n")
			sys.stdout.write("<ClientOutput>\n")

	def initLogging(self):
		self.log = GlobalConfig.log

	def initConnection(self):
		self.taskQueue = TaskQueue()
		self.setup()
		self.connect()
		
	def finish(self):
		if GlobalConfig.xmlOutput:
			sys.stdout.write("</ClientOutput>\n")
		
	def setup(self):
		Connection.setup(self)
		self.server = GlobalConfig.server

	def connectServer(self, server):
		# print server, self.port
		try:
			self.sock.connect((server, self.port))
			self.log.write("Connected to the server\n",
						   LogLevel.COMMS_EVENT)
		except socket.error, error:
			(errorNo, reason) = error
			message = 'Error: %s\n' % reason
			if GlobalConfig.xmlOutput:
				actimExit(
					formXMLLogMessage(LogLevel.ERROR, message), 5, 1)
			else:
				actimExit(message, 5)

	def handleRequest(self, request):
   	  	self.taskQueue.addTask(request)
		
	def connect(self):
		self.connectServer(self.server)

	# Main routine to send the request to the server and
	# wait for response from the server. Base on the response
	# content and see whether waiting for further response
	def run(self):

		writeSock = [self.sock]
		connMonitor = ClientConnectionMonitor(self.log)

		while 1:
			(readyToRead, readyToWrite, inError) = select([self.sock],
																writeSock,
																[], 2)
			
			if readyToRead:
				result = getRequest(readyToRead[0])

				connMonitor.updateRules(result)
				self.currentRtnCode = result.getResultStatus()
				self.processResult(result)			 

			if not readyToWrite and self.taskQueue.len():
				writeSock = [self.sock]
				continue
			else:
				writeSock = []

			if readyToWrite and self.taskQueue.len():
				task = self.taskQueue.getTask()

				connMonitor.updateRules(task)
				
				sendRequest(readyToWrite[0], task)
				self.log.write("Send a task ....", LogLevel.MINOR_EVENT)

			if connMonitor.isConnFinish():
				#self.sock.close()
				self.sock.shutdown(2)
				break
			
	def processResult(self, result):

		if result.parentType == RequestCode.AUTH:
			self.processAuthResult(result)
		if result.parentType == RequestCode.COMMAND:
		   	self.processCmdResult(result)
		elif result.parentType == RequestCode.EVENTS:
		   	self.processEventResult(result)

	# Currently, just print out the result string
	def processCmdResult(self, result):
		self.printResult(1, 'Command', result)
		
	def processAuthResult(self, result):
		self.printResult(1, 'Auth', result)
		
	def processEventResult(self, result):
		self.printResult(1, 'Event', result)
		
	def printResult(self, printParent, type, result):

		if GlobalConfig.xmlOutput:
			sys.stdout.write("<ClientResult type=\"%s\">\n" % type)
			
		if printParent:
			sys.stdout.write(result.parentObj.info(
									GlobalConfig.xmlOutput))

		sys.stdout.write(result.info())
		
		if GlobalConfig.xmlOutput:
			sys.stdout.write("</ClientResult>\n")
			
def promptForPswd(userName=None):

	if not userName:
		userName = raw_input('User email account: ')

	return userName, getpass('Password for user, '
				   			 + userName + ': ')

	
	
