from Config import *
from getpass import *
from time import *
from AllCode import *

def promptForEncryptKey():

   	while (not GlobalConfig.eKey):
		GlobalConfig.eKey = getpass('Enter encryption key: ')	

# Note that this command is only created by an Event command
# to check for authetication and check for polling before
# starting an event
def createPollEmailCmd(uid, user, password, eKey):

	args = [CommandCode.EMAIL, CommandCode.SETUP, uid, user,
			password, eKey]
	cmd = Command(CommandCode.POLL, args)

	return cmd

def pollMediaAuth(connID, uid):

	cmd = Command(CommandCode.POLL, [CommandCode.EMAIL,
									 CommandCode.REQUEST,
									 connID, uid])
	return cmd

def pollMediaCheckCmd(uid, user, passwd):

	args = [CommandCode.EMAIL, CommandCode.CHECK, uid, user, passwd]
	cmd = Command(CommandCode.POLL, args)
	return cmd
	
# Commands for daemon
class Command:

	cmdCode = None

	# If we got two exactly the same command generated
	# from the same connection. We need to somehow the
	# difference.
	id = None
	
	# This based on what the command code is.
	# When daemon receives this object, it will know
	# what to expect in the parameters
	args = None

	# xmlOutput - this xmlOutput is kept inside the object
	# because this object is transmitted to the server
	# Then the server executes the daemon command and
	# output in xml format if this option is set
	xmlOutput = None

	def __init__(self, code, args):
		self.cmdCode = code
		self.id = time()
		self.args = args
		self.xmlOutput = GlobalConfig.xmlOutput
		
	# Return match (1) if args matches self.args
	# even args is a subset of self.args still counts
	# as a match
	def checkArgs(self, code, *args):

		if code != self.cmdCode:
			return 0

		x = 0
		while x < len(args):
			if args[x] != self.args[x]:
				return 0
			x = x + 1
			
		return 1

	def info(self, xmlOut=1):
		if self.xmlOutput or xmlOut:
			return self.xmlInfo()
		else:
			return self.normalInfo()

	def normalInfo(self):
		
		name = CommandCode.codeToName[self.cmdCode]
		msg = "The command is: " + `name` + "\n"
		argsString = ""
		# print self.args
		if self.args:
			for x in self.args:
				argsString = argsString + " " + `x`
		else:
			argsString = "None"

		msg = msg + "    Parameters: " + argsString + "\n"
		return msg

	def xmlInfo(self):

		printArg = 0
		
		if self.cmdCode == CommandCode.POLL:
			printArg = 2
			
		msg = ("<Command numArg=\"%s\" name=\"%s\">\n" %
			   (printArg, CommandCode.codeToName[self.cmdCode]))
		
		count = 0

		while (count < printArg):

			if CommandCode.codeToName.has_key(self.args[count]):
				value = CommandCode.codeToName[self.args[count]]
			else:
				value = self.args[count]
  			msg = msg + ("<Arg%s>%s</Arg%s>\n" %
						 (count, value, count))
			count = count + 1
			
		msg = msg + ("</Command>\n")
		return msg
	
	def getCommandCode(self):
		return self.cmdCode

	def status(self):
		print self.info()

	# Clear sensitive information
	def clear(self):

		if (self.cmdCode == CommandCode.POLL and
			self.args[0] == CommandCode.EMAIL):

			self.args[-2:] = []

	# This routine is called by Request.check
	# Make sure the uid is present for POLL EMAIL SETUP_USER
	# Return (0, None) if the check is success, otherwise 1
	# Return (1, msgs) if error
	def check(self):

		msg = ("Command syntax error. Usage " +
	   	  	   "POLL EMAIL SETUP_USER [[user] | \"user\" \"pswd\"]\n" +
			   "POLL EMAIL CHECK [[user] | \"user\" \"pswd\"]\n")
		msg1 = ("Error. Unrecognised command key word.\n" +
				"Currently POLL command only supports:\n" +
				"POLL EMAIL SETUP_USER\nPOLL EMAIL CHECK\n"
				)
		
		if self.cmdCode == CommandCode.POLL:

			if len(self.args) < 2:
   				return (1, msg)
			
			if (CommandCode.names.has_key(self.args[0]) and
				CommandCode.names.has_key(self.args[1])):
				self.args[0] = CommandCode.names[self.args[0]]
				self.args[1] = CommandCode.names[self.args[1]]
			else:
	   			return (1, msg)

			if (self.args[0] == CommandCode.EMAIL and
				(self.args[1] == CommandCode.SETUP_USER or
				 self.args[1] == CommandCode.CHECK)):

				if len(self.args) > 4:
					return (1, msg)

				# Check what is missing and finally put everything
				# into POLL EMAIL SETUP_USER <uid> <account> <pswd> <eKey>
				
				# If the argument is POLL EMAIL SETUP_USER
				# then append with <uid> <account> <pswd> <eKey>
				
				# If the argument is POLL EMAIL SETUP_USER <account> <pswd>
				# then <uid> before <account> and append with <eKey>

				# If the argument is POLL EMAIL SETUP_USER <account>
				# then <uid> before <account> and append with <pswd> <eKey>

				# Missing <uid>, possibly missing <pswd>
				if len(self.args) > 2 and len(self.args) <= 4:
					user = self.args[2]
					GlobalConfig.emailAccount = user
					self.args[2:2] = [mdc_getuid(user)]

					# Now it's missing <pswd>
					if len(self.args) == 4:
						self.args.append(GlobalConfig.emailPassword)
					
					# <pswd> is included in the POLL .... command
					elif len(self.args) == 5:
						GlobalConfig.emailPassword = self.args[4]
						
				# Missing <uid>, 				
				elif len(self.args) == 2:
					user = GlobalConfig.emailAccount
					self.args.append(mdc_getuid(user))
					self.args.append(user)
					self.args.append(GlobalConfig.emailPassword)

				promptForEncryptKey()
				self.args.append(GlobalConfig.eKey)

				return (0, None)

		elif self.cmdCode == CommandCode.QUIT:
			return (0, None)

		# Some queue info checking should move to here
		elif self.cmdCode == CommandCode.QUEUE_INFO:
			return (0, None)
	
		elif self.cmdCode == CommandCode.SHOW_CONFIG:
			return (0, None)
	
		return (1, msg1)

			
