from os.path import *
from DaemonDft import *
from SecurityDft import *
from ClientDft import *
from ConfigParser import *
from MDC import *
import sys, string
from types import *
from copy import *
import os, re
from stat import *
from time import *

requireConfig = [
			"emailPassword", "emailAccount",
	   	    "emailAddress", "IMAP4_server",
		   	"IMAP4_port", "SMTP_server",
		   	"SMTP_port", "pollEmailInterval",
		   	"internetMedia", "mailSubject",
			"numConnection", "saveRsltDir",
		   	"saveRemoteRslt", "logLevel",
		   	"logStatus", "sendEmail",
		   	"requestResult", "resultAction",
		   	"waitResult", "compressFiles",
		   	"targetPath", "transmitMedia",
			"port", "server", "xmlOutput",
			"connection"
		   ]

requireSecurityConfig = [
			"serviceTimeLimit",
			"allowExecution", "allowFileSend",
			"allowFileReceive", "restrictedExecution"
			]

# must be defined if serviceTimeLimit is on
optionalSecurityConfig = [
			"startTime", "endTime"
			]

tempList = copy(requireConfig)
tempList.append('eKey')
tempList.extend(requireSecurityConfig)
tempList.extend(optionalSecurityConfig)

# This is to indicate when configuration error occurs
# and cannot continue. The exit error string should be
# either presented in XML or normal string is decided
# by xxxxxDft.xmlOutput
exitXMLOutput = []

# It's a generic class that creates attributes and value
# from the configuration that has whatever name and value
class GlobalConfig:
	pass

# Whatever it's not defined in user config,
# merge with default selection. Then we
# have a full set of configuration
def mergeOptions(optionDict, configParser, section):

	for x in optionDict.keys():
		if (not x in ("__doc__", "__module__") and
			not configParser.has_option(section, x)):
			
			if not configParser.has_section(section):
				configParser.add_section(section)

			configParser.set(section, x, optionDict[x])
			

def mergeDefaultOptions(optionDict, configParser):

	mergeOptions(optionDict, configParser, None)
	
# Given configParser object, this function updates
# all the option values in configParser inside the
# GlobalConfig class
def setGlobalConfig(configParser):

	for x in configParser.defaults().keys():
   		GlobalConfig.__dict__[x] = configParser.defaults()[x]

	if configParser.has_section("SECURITY"):
		
		optionList = configParser.options("SECURITY")

		# Match for any rcmdl_... option name
		rCmdLineOptName = re.compile('rcmdl_\w\w*')
		
		for x in optionList:
			# This if statement is to overcome a python bug in
			# ConfigParser. Temporary statement
			if (x in requireSecurityConfig or
			    x in optionalSecurityConfig):
				GlobalConfig.__dict__[x] = configParser.get("SECURITY", x, 1)

			# Match any option name starts with rcmdl_\w inside
			# the security option
			pMatchResult = rCmdLineOptName.match(x)

			if pMatchResult != None:
				GlobalConfig.__dict__[x] = configParser.get("SECURITY", x, 1)

	intList  = ['IMAP4_port',
				'SMTP_port',
				'pollEmailInterval',
				'numConnection',
				'saveRemoteRslt',
				'logLevel',
				'logStatus',
				'sendEmail',
				'requestResult',
				'resultAction',
				'waitResult',
				'compressFiles',
			    'transmitMedia',
				'port',
				'connection',
				'xmlOutput',
				'serviceTimeLimit',
				'allowExecution',
				'allowFileReceive',
				'allowFileSend',
				'restrictedExecution'
			   ]

	for key in tempList:

		if key in GlobalConfig.__dict__.keys():
			
			# Automatically convert the option suppose to
			# be int value but it was entered as string value
			if key in intList:
				if type(GlobalConfig.__dict__[key]) is StringType:
					try:
						updateValue = int(GlobalConfig.__dict__[key])
					except:
						errorString = ("Error: cannot convert option, " +
										 key + ". Please " +
										 "double check the configuration value\n")
						actimExit(errorString, 2)
										 
				else:
					updateValue = GlobalConfig.__dict__[key]
			else:
				updateValue = GlobalConfig.__dict__[key]

			GlobalConfig.__dict__[key] = updateValue

# This command makes sure all the options in
# GlobalConfig.__dict__ has no empty value
# If so, this will cause the program to exit
# before any further
def checkGlobalConfig(security=0):

	template = "Error: option %s has empty value\n"
	
   	# Make sure the updateValue is neither "" or None
	# if so, we cannot proceed further

	checkOptsList = copy(requireConfig)
	
	if security:
		checkOptsList.extend(requireSecurityConfig) 

	for key in checkOptsList:
	    if GlobalConfig.__dict__.has_key(key):
			updateValue = GlobalConfig.__dict__[key]

			if (updateValue == ""  or updateValue == None or updateValue == "None"):
				actimExit(template % key, 3)

	# If logStatus is on, then make sure logfile option
	# not empty
	if GlobalConfig.logStatus:
		if (GlobalConfig.__dict__.has_key("logfile") and
			GlobalConfig.logfile != None and
			len(GlobalConfig.logfile)):
			pass
		else:
			actimExit(template % "logfile", 3)
	else:
		GlobalConfig.logfile = ""
		
	# More difficult on security configuration checkings
	# If serviceTimeLimit is on, then try parsing the
	# time format in startTime and endTime
	if (GlobalConfig.__dict__.has_key("serviceTimeLimit") and
	    GlobalConfig.serviceTimeLimit == 1):

		# parseTime and workout the time 
		GlobalConfig.startTimeTuple = mdc_parseTime(GlobalConfig.startTime)
		GlobalConfig.endTimeTuple = mdc_parseTime(GlobalConfig.endTime)
		if (GlobalConfig.startTimeTuple == None or
			GlobalConfig.endTimeTuple == None):
		   	actimExit("Error: invalid time specification\n", 3)

		else:
			# Make down whether the time frame cross to the
			# next day
			GlobalConfig.crossDay = (GlobalConfig.startTimeTuple >
									 GlobalConfig.endTimeTuple)

	# If restrictedExecution is on, then we need make sure at least
	# one rcmdl_.... is defined
	if (GlobalConfig.__dict__.has_key("restrictedExecution") and
		GlobalConfig.restrictedExecution):
		
	   	# Match for any rcmdl_... option name
   		rCmdLineOptName = re.compile('rcmdl_\w\w*')
		found_rcmdl = 0
		
		for x in GlobalConfig.__dict__.keys():
			patMatch = rCmdLineOptName.match(x)
			if patMatch != None:
				rcmdl_expression = GlobalConfig.__dict__[x]

				# Make sure all the re of restricted command lines
				# are all valid
				try:
					re.compile(rcmdl_expression)
					found_rcmdl = 1
				except:
					actimExit("Error: invalid regular expression in %s\n" % x,
							  14)

		if not found_rcmdl:
			actimExit("Error: no restricted command line execution defined\n",
					  13)
			
# Show config
def showConfig(xmlOutput):

	msg = ''
	undisplayList = ['emailPassword', 'mailSubject', '__doc__',
					 '__module__', 'log', 'restrictedExecution',
					 'allowExecution', 'allowFileSend',
					 'allowFileReceive', 'serviceTimeLimit',
					 'startTime', 'endTime']

	# Also not to show any restricted command option
	rcmdlList = re.compile('rcmdl_[\w]*')
	for x in GlobalConfig.__dict__.keys():
		matchPat = rcmdlList.match(x)
		if matchPat != None:
			undisplayList.append(x)
			
	if xmlOutput:
		template = ("<Option>\n\t<Name>%s</Name>\n\t" +
					"<Value>%s</Value>\n</Option>\n")
	else:
		template = "%s = %s\n"
		
	for x in GlobalConfig.__dict__.keys():
		if not (x in undisplayList):
			msg = msg + (template % (x, GlobalConfig.__dict__[x]))
		
	return msg

# rcfile is a list of possible locations of rcfile (with path)
# if daemon is 1, generates a daemon config object
# otherwise, generates a client config object
def initConfig(rcfile, logfile, daemon):

	foundResource = 0

	for resource in rcfile:
		if os.path.isfile(resource):

			# If actim, then only allow readable to owner
			# If actimd, it can read owner readable only
			# or owned by other people, the rc file must be
			# readable only to other. No writable permission
			if not daemon:
				group = 0
				other = 0
			else:
				# Check whether the file belongs to me
				if mdc_isMyFile(resource):
					group = 0
					other = 0
				else:
					group = 4
					other = 4
					
			result = mdc_checkFilePermission(resource,
											 group, other)
					
			if (result[0]):
				actimExit(result[1], 6)
	
			daemonConfig = ConfigParser()

			# Convert it to case-sensitive
			daemonConfig.optionxform = str
			
			fp = open(resource, 'r')
			try:
				daemonConfig.readfp(fp)
			except MissingSectionHeaderError:
				sys.stderr.write("Missing section header error\n")
			except ParsingError, e:
				err = e.errors
				for x in err:
					sys.stderr.write("Parse error, line no. " +
									 `x[0]` + "\n") 
			else:
				fp.close()

			foundResource = 1
			break

 	if not foundResource:
		mdc_rcfilesWarning(daemon)
		daemonConfig = ConfigParser()

		# Make configParser case-sensitive
		daemonConfig.optionxform = str

	if daemon:
		dict     = DaemonDft.__dict__
		security = SecurityDft.__dict__
		mergeOptions(security, daemonConfig, "SECURITY")

	else:
		dict = ClientDft.__dict__

	exitXMLOutput.append(dict['xmlOutput'])
	
	# For those configurations that are not defined in rcfile
	# merge the values from the default configuration
	mergeDefaultOptions(dict, daemonConfig)

	# If the logfile is specified in the argument, then overwrite
	# the daemon configuration
	if logfile:
		daemonConfig.logfile = logfile
		daemonConfig.logStatus = 1

	setGlobalConfig(daemonConfig)

# Requires 2.0 or above to run
# Returns (0, None) - if the current python is 2.0 or above
# else returns (1, ErrMsg)
def checkPyVersion():

	# Below version 2.0 uses sys.version
	# otherwise uses sys.version_info instead
	if hasattr(sys, 'version_info'):
		versionInfo = sys.version_info
		if versionInfo[0] >= 2:
			return (0, None)

	# Don't need to check further, we said it's below 2.0
	errMsg = ("Current python version: " + sys.version + "\n" +
	   		  "Minimum requirement: Python 2.0 or above\n")
		

	return (1, errMsg)

# This is called when the current polling time cross the end of
# service time. This routine is to recalculate the time frame
# in number of secs. The function is called when
# the serviceTimeLimit is on
def updateServiceTimeFrame():

	currentNumSecs = time()
	currentTimeTuple = localtime(currentNumSecs)

	# Subsitute all the current date and current time
	tempTuple =      (currentTimeTuple[0], currentTimeTuple[1],
					  currentTimeTuple[2],
					  GlobalConfig.startTimeTuple[3],
					  GlobalConfig.startTimeTuple[4],
					  GlobalConfig.startTimeTuple[5],
					  currentTimeTuple[6], currentTimeTuple[7],
					  currentTimeTuple[8])
	
	GlobalConfig.startSecs = mktime(tempTuple)

	# If it cross to the next day, then we need to advance to the
	# next day by all total number of secs in a day to currentNumSecs
	if GlobalConfig.crossDay:

		currentNumSecs = currentNumSecs + (24 * 60 * 60)
		currentTimeTuple = localtime(currentNumSecs)

	tempTuple =      (currentTimeTuple[0], currentTimeTuple[1],
					  currentTimeTuple[2],
					  GlobalConfig.endTimeTuple[3],
					  GlobalConfig.endTimeTuple[4],
					  GlobalConfig.endTimeTuple[5],
					  currentTimeTuple[6], currentTimeTuple[7],
					  currentTimeTuple[8])
	
	GlobalConfig.endSecs = mktime(tempTuple)

# The function is to construct a XML error template
# with exit code
# This is particular useful for integration with GUI
# and crappy Windows
def xmlErrorTemplate(errorString, exitCode):

	xmlTemplate = "<?xml version=\"1.0\"?>\n"
	xmlTemplate = xmlTemplate + "<ClientOutput>\n"
	xmlTemplate = xmlTemplate + "<ClientError>\n"
	xmlTemplate = xmlTemplate + "\t<Error>%s</Error>\n" % errorString
	xmlTemplate = xmlTemplate + ("\t<ReturnCode>%s</ReturnCode>\n" %
								 exitCode)
	xmlTemplate = xmlTemplate + "</ClientError>\n"
	xmlTemplate = xmlTemplate + "</ClientOutput>\n"

	return xmlTemplate

# If under Windows, or xxxxDft.xmlOutput or
# xmlOutput in actimrc file is set. Then
# print exit error message in XML and exit
# force means override all conditions and gives
# error in xml format
def actimExit(errorString, exitCode, force=0):

	if (force or mdc_isLose() or
	    (len(exitXMLOutput) and exitXMLOutput[0] == 1)):
		errorOutput = xmlErrorTemplate(errorString, exitCode)
	else:
		errorOutput = errorString

	sys.stdout.write(errorOutput)
	sys.exit(exitCode)
