from threading import *
from pickle import *
from socket import *
from Connection import *
import select
from LogLevel import *
from Result import *
from Request import *
from Command import *
from AllCode import *
from DaemonCore import *
import sys

class DaemonConn(Connection):

	# bitwise setting of media for polling
	pollSet = None
	transmitSet = None
	
	# This is a set of MediaTransferClass objects
	mediaTransmitObjs = []

	# This is a set of MediaPollClass objects
	mediaPollObjs = []

	# The number of connection accepted
	numConnection = None

	# The core is the kernel of the Daemon
	core = None

	# list of socket connections that Daemon connects to
	readerList = []
	writerList = []

	# Keep track of all connected clients
	# connection object is the key, addr is the value
	clientList = {}

	# Local queue that a list of results are pending
	# to send back to a list of clients.
	clientResults = []

	# log is an instance of logging class
	log = None

	# Input queue and output queue. These queues are the
	# main interface for external module.
	incomingQueue = None
	outgoingQueue = None
	
	def __init__(self):
		self.log = GlobalConfig.log
		self.setup()
		self.incomingQueue = DaemonQueues.incomingTask
		self.outgoingQueue = DaemonQueues.clients_result
		self.initConn()
		self.listen()

	# confDict should contain all the possible configuration
	# variables
	def setup(self):
		
		Connection.setup(self)

		# Hard code to a single connection
		# Time constraint no time to fix
		self.numConnection = 1
		
	# direction: 1 - transmit, 0 - poll
	def getMedia(self, transmit, type):

		if transmit:
			tmpSet = self.transmitSet
			mediaObjs = self.mediaTransmitObjs
		else:
			tmpSet = self.pollSet
			mediaObjs = self.mediaPollObjs
			
		if type & tmpSet:
			for x in self.mediaTransmitObjs:
				if x.isMedia(type):
					return x
			sys.stderr.write("Strange Error: " +
							 "No media object found but has switched on\n")
		else:
			sys.stderr.write("Can't get it for this media type. " +
							 "Not configured\n")

		return None

	def getTransmitMedia(self, type):
		return self.getMedia(self, 1, type)

	def getPollMedia(self, type):
		return self.getMedia(self, 0, type)

	# Daemon starts listening on the port
	def initConn(self):
		try:
			self.sock.bind((socket.gethostname(), self.port))
		except socket.error, (num, error):
			actimExit("Error: %s\n" % error, 18)
			
		self.log.write("Listening ...\n", LogLevel.MAJOR_EVENT)
		self.sock.listen(self.numConnection)

	# Peek to see whether the data in the socket is EOF
	def isEOF(self, conn):
		if len(conn.recv(1, socket.MSG_PEEK)) == 0:
			return 1
		return 0

	def sendHelloMsg(self, conn):
		helloMsg = Result(RequestCode.COMMAND, 0, 0,
						  "Hello message from " + `gethostname()` +
						  "\n", None,
						  Command(CommandCode.names['HELLO_MSG'],
								  None)
						  )
		sendRequest(conn, helloMsg)
		self.log.write("Sent hello message to socket: " + `conn.fileno()` +
					   "\n", LogLevel.COMMS_EVENT)

	# Match connection ID against result
	# Main infinite loop to wait and accept connections
	def listen(self):

		# Put the main listening socket as one of the reader list
		self.readerList.append(self.sock)
		writerList = []
		while 1:
			(readyToRead, readyToWrite,
			 inError) = select.select(self.readerList, writerList, [], 2)

			# If readyToRead is non-empty, then check whether it is the
			# listening socket. If so, we accept the connection and adds
			# a new connection into the readerList.
			# If it is not a listening socket, that means we need to have
			# a request from a client.
			for reader in readyToRead:
				if reader == self.sock:
					self.log.write("Incoming connection ... \n",
								   LogLevel.COMMS_EVENT)
					
					conn, addr = self.sock.accept()

					self.log.write("Accept incoming conneciton on " +
								   `addr` + "\n", LogLevel.COMMS_EVENT)
					
					self.readerList.append(conn)
					self.writerList.append(conn)
					self.clientList[conn] = addr
					self.sendHelloMsg(conn)
				else:
					if self.isEOF(reader):
						self.log.write("Client socket connection closed\n",
									   LogLevel.COMMS_EVENT)
						reader.shutdown(2)
						reader.close()
						self.readerList.remove(reader)
						self.writerList.remove(reader)
						del self.clientList[reader]
	
					else:
						request = getRequest(reader)

						# Mark the request from which connection ID
						# the integral port number of a connection
						request.setConnID(self.clientList[reader][1])
						
						self.incomingQueue.addTask(request)
						self.log.write("Receive a request ... \n",
									  LogLevel.COMMS_EVENT)
					

			# If a socket is ready to send data, then we need to first
			# check whether, we have result for that connection
			if not self.outgoingQueue.len():
				writerList = []
			elif not writerList:
				writerList = self.writerList
				continue

			if readyToWrite and self.outgoingQueue.len():

				# We need to match the connection id against the connection
				# ID in the result.
				for writer in readyToWrite:
					result = self.outgoingQueue.matchFunc(matchResult,
														  self.clientList[writer][1])
					if result:
						self.log.write("Send result to connection: " +
									   `writer.fileno()` + '\n',
									   LogLevel.COMMS_EVENT)
						# print result.info()
						sendRequest(writer, result)

						# Look at the type of result. If it is QUIT
						# command, then exit gracefully.
						if (result.isCommand() and result.isSuccess() and
							result.getCommandCode() == CommandCode.QUIT):
							message = "Actim server exit gracefully\n"
							self.log.write(message,	LogLevel.MAJOR_EVENT)
							mdc_syslog(message)
							writer.close()
							self.sock.close()
							sys.exit(0)
			#elif inError:

				
				
			
			
