from time import *
from Envelope import *
from DeliveryAgent import *
from pickle import *
from rotor import *
from base64 import *
from Config import *
from AllCode import *

class Event:

	# The code name is used for matching the operation
	# on the remote side if there is one
	codeName = None
	
	# ID for this event.
	id = None
	group_id = None

	# Connection ID - which client makes this request
	connID = None

	# Source object contains either command line or
	# files data. 
	source = None

	# EXECUTE, SEND or RECEIVE
	operation = None

	# restricted mode - this is used by restrictedExecution
	# to identify the command. Print function in this
	# class doesn't print this content
	re_Or_id = None
	
	# target remote path for put mode
	# target local path for get mode
	target = None

	deliveryMethod = None

	envelope = None

	# Local or remote - Local means the event is going to
	# be sent prior execution
	# Remote means the event is receving from polling object
	# Used in event_pending to decide which queue to go to
	local = None

	# delivery data - specific data for different type of
	# delivery method on arrival from remote site.
	# E.g. the message_set corresponds to this event in SMTP
	deliveryData = None

	# the result object for this event. This is attached
	# onto the event from remote site.
	result = None

	# Symmetric encryption key
	eKey = None
	
	def __init__(self):
		self.id = time()
		self.local = 1
	
	# Setup the source, target, result, and operation
	def setEnvelope(self, target):
		self.envelope = target

	def setSource(self, source):
		self.source = source

	def setTargetPath(self, path):
		self.target = path

	def setOperation(self, operation):
		self.operation = operation

	def setCommandLineType(self, type):
		self.re_Or_id = type
		
	def setDeliveryMethod(self, method):
		self.deliveryMethod = method

	def getDeliveryMethod(self):
		return self.deliveryMethod

	def setResult(self, result):
		self.result = result

	def getSource(self):
		return self.source

	def getResult(self):
		return self.result

	def getTargetPath(self):
		return self.target

	def getOperation(self):
		return self.operation
	
	def getCommandLineType(self):
		return self.re_Or_id
		
	def hasResult(self):
		if self.result != None:
			return 1
		return 0
	
	def setCodeName(self, name):
		self.codeName = name

	def getTransmitRoute(self):
		return self.envelope.getTransmitRoute()

	def getPollMediaAccount(self):
		return self.envelope.getPollMediaAccount()

	def hasSenderPassword(self):
		return self.envelope.hasSenderPassword()

	def getSenderPassword(self):
		return self.envelope.getSenderPassword()

	def setConnID(self, connID):
		self.connID = connID

	def getConnID(self):
		return self.connID

	def setRemote(self):
		self.local = 0

	def setLocal(self):
		self.local = 1
		
	def isLocal(self):
		return self.local

	def isCommand(self):
		return self.source.isCommand()

	def isFile(self):
		return self.source.isFile()
	
	def getCommand(self):
		return self.source

	def info(self, xmlOut=0):

		if xmlOut:
			return self.xmlInfo()
		else:
			return self.normalInfo()
		
	def normalInfo(self):

		msg = ''
		
		if self.codeName:
			msg = 'Event code name is ' + self.codeName + '\n'
			
		msg = msg + 'Event id is: ' + `self.id` + '\n'
		msg = msg + 'Connection id is: ' + `self.connID` + '\n'
		msg = msg + 'Transmit method through: '
   		msg = msg + Delivery.names[self.deliveryMethod] + '\n'
		return msg

	def xmlInfo(self):

		name = ""
		id   = ""
		
		if self.operation != None:
			name = Operation.codeToName[self.operation]

		if self.id != None:
			id = self.id
			
		msg = ("<Command numArg=\"0\" name=\"%s\" id=\"%s\"></Command>\n" %
			   (name, id))

		return msg
	
	def status(self):
		print self.info()
		
	def pack(self):
		if (self.source and self.source.isFile()):
			self.source.dataToBlocks()
		self.envelope.clearPassword()
		self.eKey = None

	def unpack(self):
		if (self.source and self.source.isFile()):
			self.source.blocksToData()

	def clearSourceData(self):
		self.source.clearSourceData()

	def getEKey(self):
		return self.eKey
			  
	# First, it does the internal packing, then
	# serialize the object, and then encrpypts into
	# printable quote string
	# Returns a series of string
	def encryptToPrintable(self):
		saveKey = self.eKey
		self.pack()
		data = dumps(self)
		r = newrotor(saveKey)
		encryptString = r.encrypt(data)
		encryptPrintable = encodestring(encryptString)
		saveKey = ''
		return encryptPrintable

	# Refers to Request.check and Command.check	
	def check(self):
		return (0, None)

# Returns an event object
def printableTodecrypt(string, eKey):
	unprintableString = decodestring(string)
	r = newrotor(eKey)
	decryptString = r.decrypt(unprintableString)
	try:
		event = loads(decryptString)
	except Exception, reason:
		errMsg = "Error! Cannot unserialised message content\n"
		errMsg = (errMsg +
			"This may be caused by incorrect key for decryption\n")
		if reason != None:
			errMsg = errMsg + "Reason: " + `reason` + "\n"
		GlobalConfig.log.write(errMsg, LogLevel.BAD_ERROR)
 		return None
	event.unpack()
	return event
		
		
		
