import sys, os, re, popen2, time, string, stat

# return 1 if currently running under window
# Still need to change all other methods
def mdc_isLose():

	if sys.platform[:3] == 'win':
		return 1
	return 0

# This file contains all the routines that are machine dependent

def mdc_actimrcFiles():

	if sys.platform[:3] == 'win':
		envString = 'ACTIM'
		homePath = os.environ.get(envString)
		if homePath == None:
			homePath = 'C:\\ACTIM\\'
		rclist = [homePath+'actim.ini', '.\\actim.ini']
	else:
		envString = 'HOME'
		homePath = os.environ.get(envString)
		rclist = [homePath+'/.actimrc', '/etc/actimrc' ]

	return rclist

def mdc_actimdrcFiles():
	if sys.platform[:3] == 'win':
		envString = 'ACTIM'
		homePath = os.environ.get(envString)
		if homePath == None:
			homePath = 'C:\\ACTIM\\'
		rclist = [homePath+'actimd.ini', '.\\actimd.ini']
	else:
		envString = 'HOME'
		homePath = os.environ.get(envString)
		rclist = [homePath+'/.actimdrc' , '/etc/actimdrc' ]

	return rclist

# Gives warning when run as root
# If not run as root, returns (0, None).
# Otherwise returns (1, <error msg>)
def mdc_checkroot():
	if sys.platform[:3] == 'win':
		return (0, "Warning: You are running on Windows\n")
	else:
		if (os.geteuid() == 0 or os.getegid() == 0):
			return (1, "Error: You are running as root!\n")
		else:
			return (0, None)

# Get user id from user. On windows, uses hask key instead
def mdc_getuid(user):
	if sys.platform[:3] == 'win':
		return hash(user)
	else:
		return os.getuid()

# Since window hasn't got return code, this is implemented differently
def mdc_popen(cmdString):
	
	if sys.platform[:3] == 'win':
		fromchild, tochild, childerr = popen2.popen3(cmdString)
		# Does windows has return code?
		return fromchild.readlines(), childerr.readlines(), 0 
	else:
		executeObj = popen2.Popen3(cmdString, 1)
		return (executeObj.fromchild.readlines(),
				executeObj.childerr.readlines(),
				os.WEXITSTATUS(executeObj.wait()))

def mdc_rcfilesWarning(daemon):

	if sys.platform[:3] == 'win':
		if daemon:
			filesString = './actimd.ini or ~/actimd.ini'
		else:
			filesString = './actim.ini or ~/actim.ini'
	else:
		if daemon:
			files = 'actimdrc'
		else:
			files = 'actimrc'

		filesString = ("/etc/" + files + ", ~/" + files)

	sys.stderr.write("Waring: file not found: " + filesString + '\n')
	sys.stderr.write("Use default configuration instead\n")

# Expect timeString is in the form of HH:MM:SS and
# returns the time in tuple defined in time module
def mdc_parseTime(timeString):

	timeTuple = None
	
	if sys.platform[:3] == 'win':

		# strptime is Not available for window
		timeUnits = string.split(timeString, ':')
		if len(timeUnits) != 3:
			return None

		# Make sure it's integer types
		try:
			hour = int(timeUnits[0])
			minute = int(timeUnits[1])
			seconds = (timeUnits[2])
		except TypeError:
			return None
		except ValueError:
			return None

		# Now do the range checking
		if hour < 0 or hour > 23:
			return None
		if minute < 0 or minute > 59:
			return None
		if seconds < 0 or seconds > 59:
			return None
		
		# format into the time tuple
		zeroTimeTuple = time.localtime(0)

		timeTuple = (zeroTimeTuple[0], zeroTimeTuple[1],
					 zeroTimeTuple[2], hour, minute,
					 seconds, zeroTimeTuple[6],
					 zeroTimeTuple[7], zeroTimeTuple[8])
	
	else:
		try:
			timeTuple = time.strptime(timeString, "%H:%M:%S")
		except ValueError:
			return None

	return timeTuple
	
def mdc_syslog(message, priority=-1):

	if sys.platform[:3] == "win":

		pass
	else:

		import syslog

		if priority == -1:
			priority = syslog.LOG_INFO
			
		syslog.openlog("actimd", syslog.LOG_PID, syslog.LOG_USER)
		syslog.syslog(priority, message)
		syslog.closelog()

# Return (0, None) if the file is owner readable only
# Otherwise (1, msg)
# If group is 4, then accepts 4 or 0. Same to 'other'
# If group is 0, then accepts 0 only.
def mdc_checkFilePermission(file, group, other):

	if sys.platform[:3] == 'win':
		return (0, "Warning: Windows no file permission check\n")
	
	mode = os.stat(file)[stat.ST_MODE]
	permission = oct(stat.S_IMODE(mode))

	pString = "%s" % permission
	otherMode = int(pString[-1:])
	groupMode = int(pString[-2:-1])

	msg = None
	result = 0

	tmpMsg = (file + " file has incorrect permission.\n" +
			  "Please check the documentation\n")

	for actual, expect in ((groupMode, group), (otherMode, other)):
		if (expect == 4):
			if actual == 4 or actual == 0:
				pass
			else:
				result = 1
				msg = tmpMsg
		elif (expect == 0):
			if actual == 0:
				pass
			else:
				result = 1
				msg = tmpMsg

	return (result, msg)

# Returns 1 if the filename's uid matches the caller
def mdc_isMyFile(pathname):

	if mdc_isLose():
		return 1
	else:
		return (os.stat(pathname)[stat.ST_UID] == os.getuid())


