import imaplib, types, os, string, re
from DeliveryAgent import *
from DaemonCore import *
from time import *
from thread import *
from StringIO import *
from Event import *
from LogLevel import *
from pickle import *
from Config import *
from AllCode import *
from Result import *
from Command import *
from TransmitMedia import *

# PollMedia class is a global class that keeps track
# on any polling thread. For example, if there is
# an item in pollEmailList, it means that there is
# a thread listening on any new actim email for an
# user
class PollMedia:

	pollEmailList = []
	pollWWWList   = []
	pollFileList  = []

	# This list contains a list of re objects that
	# match for incoming command line
	# This variable is initiated by function.
	# If restrictedExecution is off, this list will
	# contain an re('.*') object.
	restrictedCmdlList = None

# Initializes PollMedia.restrictedCmdlList variable
def initRestrictedCmdlList():

	# Equals to none means haven't initialised yet
	if PollMedia.restrictedCmdlList == None:

		PollMedia.restrictedCmdlList = {}
		
		if GlobalConfig.restrictedExecution:
			rcmdlName = re.compile('rcmdl_(\w\w*)')

			# Find any option name starts with 'rcmdl_...'
			for x in GlobalConfig.__dict__.keys():
				patMatch = rcmdlName.match(x)

				# If found one, extracts the command line id
				# and regular expression syntax
				if patMatch != None:
					rcmdl_id = patMatch.group(1)
					rcmdl_reValue = GlobalConfig.__dict__[x]

					# Compile the command line re syntax
					# If alright, add it into the
					# PollMedia.restrictedCmdlList
					rcmdl_re = None
					try:
						rcmdl_re = re.compile(rcmdl_reValue)
						PollMedia.restrictedCmdlList[rcmdl_id] = rcmdl_re
					except:
						GlobalConfig.log.write(
							"Error: invalid command-line re syntax: %s\n"
							% rcmdl_reValue, LogLevel.ERROR)

		# If restrictedExecution is off, then put a wild card re
		# object which matches all the execution
		else:
			anyCommand = re.compile('.*')
			PollMedia.restrictedCmdlList = {
					".*":anyCommand
				}

# Match restricted command line execution
# Returns 1 if the command line string matches
# one of the re objects in PollMedia.restrictedCmdlList
# Otherwise returns 0
def matchCommandLineString(cmdString):

	match = 0
	for reObj in PollMedia.restrictedCmdlList.values():
		patMatch = reObj.match(cmdString)
		if patMatch != None:
			return 1
	return match

# If not found, return None
# If found, return the command line string
def matchCommandLineID(id):

	for cmdID in PollMedia.restrictedCmdlList.keys():
		if cmdID == id:
			# Returns the pattern that was compiled with the
			# re object
			return PollMedia.restrictedCmdlList[cmdID].pattern
	return None

# Main routine to execute an event
def executeEvent(event):
	result = Result(RequestCode.EVENTS, event.id, 0, None, None, event)

	# Command line task. - use popen to the run the
	# command. Return all the return messages and
	# exitCode and generate a result object and
	# put it back to the event object
	if event.isCommand():

		if GlobalConfig.allowExecution:

			# See whether restrictedExecution is set.
			# If so, then determine when the command is
			# re or id.
			validExecution = 0

			# initialize a list of regular expression
			# ready to parse
			initRestrictedCmdlList()

			cmdString = event.getCommand().getCmdString()

			# Get the command line type, re or id
			re_Or_id = event.getCommandLineType()

			if re_Or_id == Operation.RE_SYNTAX:
				passMatch = matchCommandLineString(cmdString)

			# Retrieve the command string based on the id
			elif re_Or_id == Operation.CMD_ID:
				idString= matchCommandLineID(cmdString)
				if idString != None:
					cmdString = idString
					passMatch = 1
				else:
					passMatch = 0
			else:
				passMatch = 0

			# The server either has restrictedExecution on
			# the client invokes with qualify command ID or
			# command re. Or restrictedExection is off and
			# matches everything
			if passMatch:
				# Execute the command line
				stdoutMsg, stderrMsg, rtCode = mdc_popen(cmdString)

				logMsg = 'Execute command: %s\n' % cmdString
				rtCodeMsg = 'Return status: %s\n' % rtCode
				GlobalConfig.log.write(rtCodeMsg, LogLevel.MAJOR_EVENT)
			else:
				logMsg = "Command line string doesn't qualify %s\n" % cmdString
				stderrMsg = [logMsg]
				stdoutMsg = []
				rtCode = 15
				
   			GlobalConfig.log.write(logMsg, LogLevel.MAJOR_EVENT)
   			mdc_syslog(logMsg)
			
		else:
			stdoutMsg = ["Command-line execution is not allowed\n"]
			stderrMsg = []
			rtCode = 8
			
		result.extendMessageList(stdoutMsg)
		result.extendMessageList(stderrMsg, 0)
		result.setResultStatus(rtCode)

	# First that we need to know which operation it is
	# (put or get). Then 
	elif event.isFile():

		op = event.getOperation()
		msgList = []
		errorList = []
		source = event.getSource()

		# Extract the source from the event and then dump
		# source content into the targetPath
		if op == Operation.SEND:
			if GlobalConfig.allowFileSend:
				targetPath = event.getTargetPath()

				# Check the target path whether this is valid
				# If not, create a result object with reason
				if not os.access(targetPath, os.W_OK):
					rtCode = 1
					errorList.append('Error: Target path is not valid\n')
				else:
					rtCode = 0
					for sourceData in source.getFiles():
						rslt, msg = sourceData.dumpFile(targetPath)
						if rslt:
							rtCode = rslt
							errorList.append(msg)
						else:
							msgList.append(msg)
						GlobalConfig.log.write(msg, LogLevel.MINOR_EVENT)
						mdc_syslog(msg)
			else:
				rtCode = 11
				msgList = ["Files are not allowed to be accepted\n"]

			result.setResultStatus(rtCode)
			result.extendMessageList(msgList)
			result.extendMessageList(errorList, 0)

			# Now we got the result. The event will be put into
			# a queue and eventually send it across through
			# internet. For performance issue, the source data
			# should be clear, as no use anymore.
			event.clearSourceData()

		# Both remote and local will come to this block of code
		# put it into the source object.
		elif op == Operation.RECEIVE:

			# For executing remotely is to look at the list of files
			# specified in the source, and retrieve the content and
			if not event.getResult():

				if GlobalConfig.allowFileReceive:
					rtCode, tmpList = source.validateSource()

					# Since validateSource returns 1 means successful
					# this needs negate the result status for Result object
					if rtCode == 1:
						rtCode = 0
						msg = 'File(s) is(are) successfully retrieved'
						msgList = [msg]
						mdc_syslog(msg)
						result.extendMessageList(msgList)
					else:
						rtCode = 12
						errorList = tmpList
						result.extendMessageList(errorList, 0)
				else:
					msgList = ['Files are not allowed to be retrieved\n']
					result.extendMessageList(msgList)
					rtCode = 10

				result.setResultStatus(rtCode)
				
			# For executing locally is to put the retrieved file(s)
			# into local path
			elif event.getResult().isSuccess():

				rtCode = 0
				targetPath = event.getTargetPath()
				for sourceData in source.getFiles():
					rslt, msg = sourceData.dumpFile(targetPath)
					if rslt:
						rtCode = rslt
						errorList.append(msg)
						result.extendMessageList(errorList, 0)
					else:
						msgList.append(msg)
						result.extendMessageList(msgList)

					GlobalConfig.log.write(msg, LogLevel.MINOR_EVENT)

				result.setResultStatus(rtCode)

	event.setResult(result)

	# Now we need to send the event back to the originator
	deliverEvent(event)

# Given and event object,  it
# look at the event media operation and then transmit the
# event by object serialization
def deliverEvent(event):

	# Event can be in different status
	# 1. event that is generated locally and send to remote site
	# 2. event that is generated locally, sent to remote site
	#    and return back with result (not None)
	# 3. event that is generated remotely. (this will go into
	#    event_execute queue)
	# 4  (Consequence of 3). event that is generated remotely
	#    and now contains result. Need to send it back to the
	#    originator
	eventResult = event.getResult()

	if ((event.isLocal() and not eventResult) or
		(not event.isLocal() and eventResult)):
		GlobalConfig.log.write('Recieve event without result or an remote event ' +
							   'just been executed and contains result\n',
					   			LogLevel.COMMS_EVENT)
		
		# The route depends on whether the event is created locally
		# or remotely. For remote, needs to revert the address.
		route = event.getTransmitRoute()
		if event.isLocal():
			fromAddr, returnTo, to = route
		else:
			to, returnTo, fromAddr = route
			
		if (Delivery.SMTP ==
			event.getDeliveryMethod()):

			# Check whether the SMTP is allowed to use
			if GlobalConfig.sendEmail:

				setupSMTP()

				# If this event is generated locally, then the
				# pollmedia is needed to make sure exists or
				# create successfully, before sending the event
				if ((event.isLocal() and startPollMediaForEvent(event)) or
					(not event.isLocal())):
					data = event.encryptToPrintable()
					TransmitMedia.smtp.transmit(fromAddr, to, data)

			else:
				GlobalConfig.log.write('Event transmitting via email is not configured\n',
								  	    LogLevel.BAD_ERROR)
				# Create error result

	if event.isLocal() and eventResult:
		GlobalConfig.log.write('Recieve event with result. Forward to raw_result queue\n',
					  			LogLevel.COMMS_EVENT)
		DaemonQueues.raw_result.addTask(eventResult)

# Given an event object. This routine is to start
# up polling media for this event.
# This function simply returns 1 (exists or started)
# or 0 (fail, can't start)
def startPollMediaForEvent(event):

	error = ''
	output = ''
	
	# First need to find out the delivery method
	mediaMethod = event.getDeliveryMethod()

	# Now we create the command obj to call searchPollMedia
	uid, user = event.getPollMediaAccount()

	if (mediaMethod == Delivery.SMTP):
		cmdObj = createPollEmailCmd(uid, user, 
									event.getSenderPassword(),
									event.getEKey())
	else:
		sys.stderr.write("Not supported\n")
		sys.exit(1)

	# Search for an existing polling media that matches
	# the user account and password
	(result, pollObject, msg) = searchPollMedia(cmdObj, 1)

	# Now creates a result object and forward to the client
	if pollObject != None:
		returnCode = 0
		output = msg
	else:
		returnCode = 1
		error  = msg

	# Very important, need to clear all the password
	cmdObj.clear()

	resultObj = Result(RequestCode.AUTH, 0, returnCode, output, error, cmdObj)

	# By pass matching for matching the object in submitted queue
	# Ask this cmdObj is generated internally
	resultObj.submission = 0
	resultObj.setConnID(event.getConnID())

	DaemonQueues.raw_result.addTask(resultObj)

	return not returnCode

# Given an event object, the funciton looks inside
# the operation mode, user password inside the event. 
# Then matches for any existing polling threads.
# If found one, then returns that polling object
# If not, then create one
# Returns in a tuple
#
# If create = 1, there are two possible returns
# (1, pollObject, msg) -> It does exists, existing object that matches
# (0, pollObject, msg) -> It doesn't exist, create polling  object

# If create = 0, three possible results including both above
# (0, None, msg)
#
# This function will update the PollMedia.pollXXXXList
def searchPollMedia(cmdObj, create=1):

	foundPoll = None

	result, dummyA, dummyB, msg = parsePollCommand(cmdObj)

	# command syntax error	
	if result != 0:
		return (0, None, msg)

	type = getMediaType(cmdObj)

	pollList = []
	if type == CommandCode.EMAIL:
		pollList = PollMedia.pollEmailList
	# elif .....

	for x in pollList:
		if x.matchAccess(cmdObj):
			foundPoll = x
			break

	if foundPoll:
		msg = 'Media polling is already existed\n'
		return (1, foundPoll, msg)

	# Not found, create a new polling object
	elif create:
		(newPolling, msg) = createPolling(cmdObj)

		if newPolling != None:	
			pollList.append(newPolling)

		return (0, newPolling, msg)

	else:
		msg = "Either password is incorrect or polling doesn't exist\n"
		return (0, foundPoll, msg)

# Given a command object, parsePollCommand looks at
# the content inside the command object and then
# see the command code is correct
# Results are represented in tuple
# (return code, media type, media command, msg)
# return code is 0 if the command is correct
# otherwise 1
def parsePollCommand(task):

	returnCode = 1
	mediaType = None
	mediaCmd  = None
	msg = ''
	
	if len(task.args) < 2:
		msg = "Error: Expect POLL <MEDIA> <Cmd> [args ...]\n"
		
	elif task.args[0] == CommandCode.EMAIL:

		mediaType = task.args[0]
		mediaCmd = task.args[1]

		if (task.args[1] == CommandCode.SETUP_USER or
			task.args[1] == CommandCode.CHECK or
			task.args[1] == CommandCode.SETUP):

			# At this stage, the arguments after SETUP_USER or
			# CHECK should be well formed
			if len(task.args[2:]) != 4:
				msg = ("Error: Expect POLL EMAIL SETUP_USER <uid> <user> <password>\n")
			else:
				returnCode = 0
		else:
			GlobalConfig.log.write('Error unexpected command: ' + task.args[1] + '\n',
						   LogLevel.BAD_ERROR)

		return (returnCode, mediaType, mediaCmd, msg)

# Returns the media type inside a command object
def getMediaType(cmdObj):
	result = parsePollCommand(cmdObj)
	if result[0] == 0:
		return result[1] 

def getMediaCmd(cmdObj):
	result = parsePollCommand(cmdObj)
	if result[0] == 0:
		return result[2]
	
# Returns command object login info
def getLoginInfo(cmdObj):
	return cmdObj.args[2:]

# Returns a tuple, (poll media object, msg)
# Extract the login information from the cmdObj
# then create poll media object and the msg
# If fails, returns (None, errorMsg)
def createPolling(cmdObj):

	(result, media, a, b) = parsePollCommand(cmdObj)

	pollMedia = None

	if media == CommandCode.EMAIL:
		pollMedia = PollEmail()
		# print cmdObj.info()
		pollMedia.setup(getLoginInfo(cmdObj))
		mediaResult = pollMedia.connect()

		if mediaResult[0] == 'OK':
			start_new_thread(pollMedia.startPolling, ())

		# Make sure we don't want to keep the pollEmail
		# object if the login/startup is failed
		else:
			pollMedia = None

		# Construct the login result message to the mail
		# server
		msg = mediaResult[0] + ' ' + mediaResult[1][0] + '\n'
		
	return (pollMedia, msg)

# executeServerCommand - executes command object and puts the
# result into client_result queue
def executeServerCommand(commandObj):

	returnCode = 1
	error = None
	output = None
	xmlOutput = commandObj.xmlOutput
	
	# Get the queue status
	if commandObj.cmdCode == CommandCode.QUEUE_INFO:
		if len(commandObj.args) == 0:
			returnCode = 15
			error = "Error: Expect queue name"
		else:
			if xmlOutput:
				output = queueNameStatusXML(commandObj.args[0])
			else:
				output = queueNameStatus(commandObj.args[0])

			returnCode = 0
			if not output:
				error = ("Error: queue name: " + commandObj.args[0] +
					   " not found\n")
				returnCode = 15

	# Show all the current daemon configuration
	elif commandObj.cmdCode == CommandCode.SHOW_CONFIG:
		output = showConfig(xmlOutput)
		returnCode = 0

	elif commandObj.cmdCode == CommandCode.FETCH_RESULT:
		if len(commandObj.args) != 2:
			error = "Error: Expect FETCH_RESULT <name> <connID>"
		else:
			# Match those results in hold_result queue
			# with the user name. If correctly matched
			# set the connection ID to those result
			# and forward to the client_result queue
			while 1:

				hold_result = DaemonQueues.hold_result.getSpecificTask(('_Result__user',
																	 commandObj.args[0]))
				if hold_result == None:
					break

				GlobalConfig.log.write('Retrieve a hold result and send to the user\n',
							   LogLevel.MINOR_EVENT)

				hold_result.setConnID(commandObj.args[1])

				# Re-parent the id
				hold_result.setID(commandObj.id)
				DaemonQueues.clients_result.addTask(hold_result)

			# When it reaches here, all the hold results are
			# retrieved, and forward to the bottom return
			# result statement
		
	# Request setting up to poll for email message
	# Request setting up to check whether there is a polling
	# 	for email messages
	elif commandObj.cmdCode == CommandCode.POLL:

		mediaType = getMediaCmd(commandObj)

		create = 0
		
		if mediaType == CommandCode.CHECK:
			create = 0
		# Must be SETUP_USER here
		else:
			create = 1
			
		(returnCode, pollMedia, msg) = searchPollMedia(commandObj, create)

		if pollMedia == None:
			returnCode = 1
			error = msg
		else:
			returnCode = 0
			output = msg

		commandObj.clear()

	elif commandObj.cmdCode == CommandCode.QUIT:

		# Send a command (without submission) to result thread
		# make sure to change all the results in hold_results
		# queue to save results and reforward to the raw_results
		# queue

		# Allow to quit only if no media polling
		
		# What should we do in multi-user mode, as this application
		# is not supposed to have a root user

		# Temporary code
		returnCode = 0
		output = "Actim server exit gracefully\n"

	# Create Result object and forward to the raw_result queue							
	rslt = Result(RequestCode.COMMAND, commandObj.id, returnCode, output,
				  error, commandObj)
	DaemonQueues.raw_result.addTask(rslt)

# By giving a queue name, the function returns the 
# queue information in string
def queueNameStatus(name):

	result = ''
	
	if DaemonQueues.queueNames.has_key(name):
		queue = DaemonQueues.queueNames[name]
		result = ("There is(are) " + `queue.len()` +
					" item(s) in " + name + "\n")
		
	elif name == 'ALL':
		for x in DaemonQueues.queueNames.keys():
			queue = DaemonQueues.queueNames[x]
			result = result + ("There is(are) " + `queue.len()` +
							   " item(s) in " + x + "\n")

	return result			

def queueNameStatusXML(name):

	result = ''
	
	if DaemonQueues.queueNames.has_key(name):
		queue = DaemonQueues.queueNames[name]
		result = ("<Queue length=\"%s\">%s</Queue>\n" %
				  (queue.len(), name))
		
	elif name == 'ALL':
		for x in DaemonQueues.queueNames.keys():
			queue = DaemonQueues.queueNames[x]
			result = result + ("<Queue length=\"%s\">%s</Queue>\n" %
				  (queue.len(), x))

	return result			

# This class is responsible for polling email
class PollEmail:

	media  = None
	finishPoll = None
	
	def __init__(self):

		self.log = GlobalConfig.log
		self.msgTitle = GlobalConfig.mailSubject
		self.pollSecs = GlobalConfig.pollEmailInterval
		self.server = GlobalConfig.IMAP4_server
		self.port   = GlobalConfig.IMAP4_port
		
	def setup(self, arg):

		self.uid = arg[0]
		self.user = arg[1]
		self.__pswd = arg[2]
		self.eKey = arg[3]

	# Returns (status, [msg])
	def connect(self):

		try:
			self.media = imaplib.IMAP4(self.server, self.port)
			result = self.media.login(self.user, self.__pswd)
			self.media.select()
		except:
			# Purpose of this is to pattern match the OK result
			# from the imap login command
			result = ('ERROR', ['LOGIN faiilure'])

		return result

	def disconnect(self):
		self.media.close()
		self.media.logout()
		
	# This routine is continuously called by startPolling
	def searchMail(self):
		searchString = '(UNSEEN SUBJECT "' + self.msgTitle +'")'
		try:
			result = self.media.search(None, searchString)
		except imaplib.IMAP4.error, reason:
			result = ('NOT OK', 'Search fail: %s\n' % reason)
		except imaplib.IMAP4.abort, reason:
			result = ('NOT OK', 'Search fail and abort: %s\n' % reason)
		except:
			result = ('NOT OK', 'Unexpected error\n')

		return result
	
	def markMailDelete(self, num):
		return self.media.store(num, '+FLAGS', '\Deleted')


	# The result of python imap fetch returns a list of tuples and strings
	# This list can be in any order.
	# E.g. It can be ['929 (FLAGS ...)', ('929 (BODY[1]', content), ')']
	# Or             [('929 (BODY[1]', content), ' FLAGS ...))']
	def extractContent(self, part, content):

		for item in content:
			if type(item) == types.StringType:
				pass
			
			# First see whether the first part contains the part string
			# that we want
			if type(item) == types.TupleType:
				if item[0].find(part) >= 0:
					return item[1]
				else:
					pass

		return None
		
	# Get the header of the mail
	def parseMail(self, num):

		part = 'BODY[1]'
		
		result, content = self.media.fetch(num, '(' + part + ')')
		if result == 'OK':
			
			# The result from imaplib is a list of intermixed tuples
			# and string. Special care need to extract the real content
			# Second element in the tuple is the real content
			content = self.extractContent(part, content)
			
			self.log.write('Fetch mail ' + `num` + ' OK, with ' +
						   `len(content)` + ' bytes\n',
						   LogLevel.COMMS_EVENT)

			# Strangely this content contains \r on all the
			# end of lines. This gives problem when
			# unserialized the string to object
			content = string.replace(content, '\r', '')
			# f = open('/tmp/recieve', 'w')
			# f.write(content)
			# f.close()
			event = printableTodecrypt(content, self.eKey)
			if event:
				event.eKey = self.eKey
				event.deliveryData = num

			return event
		else:
			self.log.write('>>> Error: ' + `result` + '\n' + `content`,
						   LogLevel.COMMS_EVENT)
			return None

	# Given a list of mail numbers in string format
	# Go through a list and then return those email content
	# and parse them into event object
	def executeMails(self, list):

		# item could be string, None, or ''
		for item in list:
			if item:
				num = int(item)

				# Get the mail content and unserialised to event
				# object
				event = self.parseMail(num)

				# If it is a valid event and doesn't contain result,
				# that means this is a remote request
				if event and not event.hasResult():
					self.log.write('PollMedia receives event request\n' +
								   'Execute event\n',
								   LogLevel.COMMS_EVENT)
					event.setRemote()
					start_new_thread(executeEvent, (event,))

				# If this contains result that means this request is
				# generated originally in here and the result has
				# arrived back. 
				elif event and event.hasResult():
					self.log.write('PollMedia receives event contains result\n',
								   LogLevel.COMMS_EVENT)
					event.setLocal()

					# Certain criteria need to address. Extra work that
					# need to be done is for those events in files mode
					# that with get operation. Need to put those files
					# retrieved remotely in the local path
					if (event.isFile() and
						event.getOperation() == Operation.RECEIVE and
						event.getResult().isSuccess()):
						start_new_thread(executeEvent, (event, ))
					else:
						start_new_thread(deliverEvent, (event, ))
					
				else:
					self.log.write('Unexpected error in find incoming email status\n',
								   LogLevel.BAD_ERROR)
					if event != None:
						self.log.write(event.info(), LogLevel.BAD_ERROR)
					
				# Mark and delete
				result, statusString = self.markMailDelete(num)
				if result == 'OK':
					logMsg = 'OK: Message mark delete successfully\n'
					result, statusString = self.media.expunge()
					if result == 'OK':
						logMsg = logMsg + 'OK: Message deleted successfully\n'
					else:
						logMsg = logMsg + 'Error: Message cannot be deleted\n'
				else:
					logMsg = 'Error: cannot mark message deleted\n'
				self.log.write(logMsg, LogLevel.MINOR_EVENT)
					
	# Main routine for polling for actim emails
	# If arrives, then calls executeMails
	def startPolling(self):

		# Before start polling for any reply
		# Need to check serviceTimeLimit is set. If so,
		# then we need to initate the startSecs and endSecs
		# time frame which represent the service time
		if GlobalConfig.serviceTimeLimit:
			updateServiceTimeFrame()

		# This is to record whether we have update the new
		# time frame
		update = 0
		start  = 0
		
		while 1:

			sleep(self.pollSecs)

			if GlobalConfig.serviceTimeLimit:

				# Record the localtime in secs
				currentSecs = time()
				
				# Check whether the current time is within
				# the service time frame
				if (currentSecs >= GlobalConfig.startSecs and
					currentSecs <= GlobalConfig.endSecs):

					# Go on to do the following actions
					# we are inside the time frame. Update
					# is needed when we come to the end
					if not start:
						logMsg = "Start listening for incoming requests\n"
						mdc_syslog(logMsg)
						start = 1
						GlobalConfig.log.write(logMsg, LogLevel.MINOR_EVENT)
						
					update = 0

				# Check whether the current time has just pass
				# the end of service time frame
				elif (currentSecs > GlobalConfig.endSecs):

					# Yes, we just pass it
					if not update:
						updateServiceTimeFrame()
						update = 1

					if start:
						logMsg = "Service time limit reaches. Stop service"
						mdc_syslog(logMsg)
						start = 0
						GlobalConfig.log.write(logMsg, LogLevel.MINOR_EVENT)
						
					# otherwise the new time frame is already updated
					# out of the time frame. No action is allowed
					continue

				else:
					continue
			
			if self.finishPoll:
				self.disconnect()
				break
			
			cmdStatus, numList = self.searchMail()
			if cmdStatus == 'OK':
				# self.log.write('Search mail OK\n', LogLevel.COMMS_EVENT)

				# numList is a list containing one element and this
				# element is a string that may contain multiple items
				# separated by white space
				numList = numList[0]
				numList = string.split(numList)
				
				self.executeMails(numList)

			# If problem arise, numList is a string which is the reason
			elif cmdStatus == 'NOT OK':
				self.log.write(numList, LogLevel.BAD_ERROR)
				self.log.write('Try to restart the imap connection\n',
							   LogLevel.BAD_ERROR)
				self.disconnect()
				self.connect()
				
	# Use for finding whether a polling action exists
	def matchAccess(self, cmdObj):

		args = cmdObj.args[2:]

		if len(args) >= 3:
			return (self.uid == args[0] and self.user == args[1] and
					self.__pswd == args[2])
		return 0
	
