from AllCode import *
from Config import *

class Result:

	# identifies the type of result based on parent
	parentType = None
	parentID = None

	# Connection ID - this field is needed when
	# Request.requestResult = 1 and
	# Request.resultAction = RESULT_BACK
	connID = None
	
	# optional if the result wants to attach what exactly
	# the invoker is. This is determined by parentType
	# Currently this should be command obj or event obj
	parentObj = None
	
	# returnCode - 0, successful
	# returnCode > 0, fail
	returnCode = None

	# return message
	msgList = None
	outputMsgList = None
	errorMsgList  = None
	
	# any kind of result objects (eg. File, list of files,
	# string, 
	type = None
	dataObj = []

	# no submittion
	# If this is 1, that means the request is put into the
	# submit queue and wait for this result back to the
	# raw_result queue. If this is 0, it means when it
	# arrives to the raw_result queue, it straight off
	# to the client_result queue
	submission = None

	def __init__(self, ptype, id, rtCode, outputMsg, errorMsg,
				 parentObj=None):
		self.parentType = ptype
		self.parentID = id
		self.returnCode = rtCode
		self.outputMsg = []
		self.errorMsg = []
		if outputMsg:
			self.outputMsg.append(outputMsg)
		if errorMsg:
			self.errorMsg.append(errorMsg)
		self.parentObj = parentObj
		self.submission = 1
		self.__pswdSet = 0

	def setClientConn(self, request):
		self.setConnID(request.connID)

	def setConnID(self, connID):
		self.connID = connID

	def setID(self, id):
		self.parentID = id
		
	def setResults(self, type, dataObj): 
		self.type = type
		self.dataObj = dataObj

	def addMessage(self, msg, output=1):
		if output:
			self.outputMsg.append(msg)
		else:
			self.errorMsg.append(msg)

	def setMessageList(self, msg, output=1):
		if output:
			self.outputMsg.append(msg)
		else:
			self.errorMsg.append(msg)

	def extendMessageList(self, msg, output=1):
		if output:
			self.outputMsg.extend(msg)
		else:
			self.errorMsg.extend(msg)
		
	def info(self):
		if GlobalConfig.xmlOutput:
			return self.xmlInfo()
		else:
			return self.normalInfo()
			
	# Generate normal output message of the information
	# inside the result object
	def normalInfo(self):
		msg = ">>>> Start of result\n"
		msg = msg + ("Result code is " + `self.returnCode` + "\n")
		msg = msg + ('Result type: %s\n' %
					 RequestCode.codeToName[self.parentType])
		
		msg = msg + 'Return message is ... \n'

		msg = msg + '---> Start of output messages\n'
		if self.outputMsg:
			for x in self.outputMsg:
				msg = msg + x
		msg = msg + '<--- End of output messages\n'
				
		msg = msg + '---> Start of error messages\n'
		if self.errorMsg:
			for x in self.errorMsg:
				msg = msg + x
		msg = msg + '<--- End of error messages\n'

		#if self.connID:
		#	msg = (msg + "Connection ID is " + `self.connID` +
		#		   "\n")

		msg = msg + "<<<< End of result\n"
		return msg

	# Generate xml output message of the information
	# inside the result object
	def xmlInfo(self):
		msg = "<Result>\n"
		msg = msg + ("\t<ReturnCode>%s</ReturnCode>\n" % self.returnCode)
		msg = msg + ("\t<Type value=\"%s\">%s</Type>\n" %
				   (self.parentType, RequestCode.codeToName[self.parentType]))

		msg = msg + ("\t<Output>")
		if self.outputMsg:
			for x in self.outputMsg:
				msg = msg + x
		msg = msg + ("</Output>\n")
		
		msg = msg + ("\t<Error>\n")
		if self.errorMsg:
			for x in self.errorMsg:
				msg = msg + (x)
		msg = msg + ("\t</Error>\n")

		msg = msg + ("</Result>\n")
		return msg

	def isCommand(self):
		return self.parentType == RequestCode.COMMAND

	def makeFileName(self):
		filename = ''
		if self.parentType == RequestCode.COMMAND:
			filename = 'cmd.'
		else:
			filename = 'events.'
		
		filename = filename + `int(self.parentID)`
		return filename
	
	def getCommandCode(self):
		return self.parentObj.cmdCode

	def getCommand(self):
		return self.parentObj
	
	# Match whether the result is correspond to request
	def matchCommandID(self, request):

		if self.parentID == request.id:
			return 1
 		
		return 0

	def isRequest(self):
		return 0
	
	def isSuccess(self):
		return self.returnCode == 0

	def setResultStatus(self, status):
		self.returnCode = status

	def getResultStatus(self):
		return self.returnCode

	# Soon with password
	def setUser(self, user):
		self.__user = user

	def checkUser(self, user):
		if self.__user == user:
			return 1
		return 0
