import os
from popen2 import *
import copy
from os.path import *
from DeliveryAgent import *
from TaskQueue import *
from Request import *
from Result import *
from thread import *
from LogLevel import *
from pickle import *
from Event import *
from Command import *
from PollMedia import *
from MDC import *
from Config import *
from DaemonCore import *
from AllCode import *

# This class sits inside the DaemonCore class and acts
# as a plugin module. A DaemonCore object should be
# able to support more than one TaskManager object for
# support different kind of queues.

# TaskManager - All it does is to move tasks around the
# queues
class TaskManager:
	
	inQueue = None
	outQueue = None

	def __init__(self, inQ, outQ):
		self.inQueue = inQ
		self.outQueue = outQ

	def start(self):
		start_new_thread(self.run, ())
		
	def run(self):
		while 1:
			inTask = self.inQueue.getTask()

			doneTask = self.processTask(task)

			self.outQueue.addTask(doneTask)


# Divert task in raw_result queue to clients_result or
# hold_result queue by referencing associated request
# in submitted queue
class ResultTaskManager:
	
	submitted = None
	hold_result = None
	raw_result = None
	clients_result = None

	def __init__(self):
		start_new_thread(self.run, ())
		
	# Any result in the raw_result has to do checking
	# against tasks in the submitted queue to find the
	# right pair.
	# Assume right request in submitted queue found that
	# match the result in raw_result.
	# Then perform the following criteria:
	# - submitted task should specify whether the raw result
	#   is returned back to the user, disgard, or hold
	def run(self):

		while 1:
			result = DaemonQueues.raw_result.getTask()

			# The result's submission is set. It means
			# that the task generates the
			# result is originally put into the submitted
			# queue
			if not result.submission:
				DaemonQueues.clients_result.addTask(result)
			else:

				# Note this function has already taken out the request
				# from submitted queue if match is found
				matchRequest = DaemonQueues.submitted.matchFunc(result.matchCommandID)

				# if no request match, then disgard the request and put the
				# result into hold
				if not matchRequest:
					DaemonQueues.hold_result.addTask(result)
					GlobalConfig.log.write('Cannot find a matched request to the submitted\n',
								   LogLevel.BAD_ERROR)
					# and produce a warning

				elif not matchRequest.requestResult:
					GlobalConfig.log.write('Result is not requested to send back. Leave it\n',
								   LogLevel.MINOR_EVENT)
					pass

				elif matchRequest.resultAction == ResultCode.RESULT_BACK:
					GlobalConfig.log.write('Put result in to the clients_result queue\n',
								   LogLevel.MINOR_EVENT)
					
					result.setClientConn(matchRequest)
					DaemonQueues.clients_result.addTask(result)

				elif matchRequest.resultAction == ResultCode.HOLD_RESULT:
					GlobalConfig.log.write('Result is configured to put in hold queue\n',
								   LogLevel.MINOR_EVENT)

					# For events results, we need to mark these results with
					# users. At the moment, there is no security. Improve later
					if matchRequest.isEvent():
						event = matchRequest.getEvent()
						result.setUser(event.envelope.getAccountName())

					DaemonQueues.hold_result.addTask(result)

				elif matchRequest.resultAction == ResultCode.SAVE_RESULT:
					GlobalConfig.log.write('Result is configured to save into a file\n',
								   LogLevel.MINOR_EVENT)

					saveDir = GlobalConfig.saveRsltDir
					if os.access(saveDir, os.W_OK):
						filename = join(saveDir, result.makeFileName())

						f = os.open(filename, os.O_RDWR | os.O_CREAT)
						os.write(f, result.info())
						os.close(f)
						GlobalConfig.log.write('Save result into ' + filename + '\n',
									   LogLevel.MINOR_EVENT)
				else:
					GlobalConfig.log.write('Oh dear! Should never reach here\n',
								   LogLevel.BAD_ERROR)
					
					# Should never reach here
					pass


# This class is reponsible for processing any objects inside the
# incomingTask queue  
class EventTaskManager:

	def __init__(self):
		start_new_thread(self.run, ())

	def run(self):

		while 1:

			# Whatever object comes out from the incomingTask queue
			# is an instance of Request class.
			request = DaemonQueues.incomingTask.getTask()

			cmdObj = request.getCommand()

			DaemonQueues.submitted.addTask(request)

			# The request is internal command for server only
			# It's a Command object
			if request.type == RequestCode.COMMAND:

				if cmdObj.cmdCode == CommandCode.FETCH_RESULT:
					cmdObj.args.append(request.connID)

				start_new_thread(executeServerCommand, (cmdObj, ))
			
			# It's an event object
			elif request.type == RequestCode.EVENTS:

				start_new_thread(deliverEvent, (cmdObj, ))

