# Make sure to do import threading rather than
# from threading import *, because this may conflict
# with Event class in the threading module
import threading
from time import *

class TaskQueue:

	# Conditional variable
	cv = None
	queueLock = None

	name = None
	
	tasks = None

	def __init__(self, name=None):
		self.name = name
		self.queueLock = threading.Lock()
		self.cv = threading.Condition(self.queueLock)
		self.tasks = []
		
	def len(self):
		self.queueLock.acquire()
		length = len(self.tasks)
		self.queueLock.release()
		return length

	def addTask(self, task):
		self.cv.acquire()
		self.tasks.append(task)
		self.cv.notify()
		self.cv.release()

	def getTask(self):
		self.cv.acquire()
		while not len(self.tasks):
			self.cv.wait()
		task = self.tasks.pop(0)
		self.cv.release()
		return task

	# fun is expected to take a task arg and
	# return 0 or 1
	def matchFunc(self, func, *args):
		foundTask = None

		self.cv.acquire()
		for task in self.tasks:
			if ((args and func(task, args)) or
				(not args and func(task))):
				self.tasks.remove(task)
				foundTask = task
				break
		self.cv.release()
		return foundTask

	def status(self, msg=None):
		if msg:
			print msg
		if self.name:
			print "Name: " + self.name
		print self.tasks
	
	# The first one is the time out
	# the rest is in name and value pair
	# The function will return when a task is found
	def getSpecificTaskWait(self, *args):
		waitSecs = args[0]
		newArgs = args[1:]
		
		self.cv.acquire()
		while 1:
			found = 0
			for task in self.tasks:
				if not len(args):
					return None
				for name, value in newArgs:
					if hasattr(task, name):
						if getattr(task, name) == value:
							found = 1
							break
				if found:
					self.tasks.remove(task)
					self.cv.notify()
					self.cv.release()
					return task
				
			if not found:
					self.cv.wait(waitSecs)
				

	def getSpecificTask(self, *args):

		self.cv.acquire()
		if not len(self.tasks):
			self.cv.release()
			return None

		else:
			for task in self.tasks:
				found = 0
				for name, value in args:
					if hasattr(task, name):
						if getattr(task, name) == value:
							found = 1
							break
				if found:
					self.tasks.remove(task)
					self.cv.release()
					return task

			self.cv.release()
			return None
