from MDC import *
from Config import *
from testTools import *
from time import *
from ParseResult import *
import os, imaplib
import popen2
import rfc822
import types

numPassed = 0
numFailed = 0

defaultOpt = {
	"eKey":"testing"
}

if mdc_isLose():
	sys.exit(0)
	
backUpActimrc()

modifyActimRC(mapPath("~/.actimrc"), defaultOpt, None)

if mdc_isLose():
	initConfig([mapPath("~/.actimrc")], None, 0)
else:
	initConfig([expanduser("~/.actimrc")], None, 0)

emailAddress = GlobalConfig.emailAddress
user         = GlobalConfig.emailAccount
pswd         = GlobalConfig.emailPassword

if mdc_isLose():
	initConfig([mapPath("~/.actimdrc")], None, 0)
else:
	initConfig([expanduser("~/.actimdrc")], None, 0)

mailSubject = GlobalConfig.mailSubject
imapServer  = GlobalConfig.IMAP4_server
port        = GlobalConfig.IMAP4_port

command = "python ../../actim -E -e 'date +\"%s\"' -d " + emailAddress + " -R --xmlOut"

# 2 minutes later from now
later = float(time() + 1 * 60)
end   = float(later + 3 * 60)

# Now send the time to startTime to endTime
startTime = strftime("%H:%M:%S", localtime(later))
endTime = strftime("%H:%M:%S", localtime(end))

sys.stdout.write("Start listening at %s\n" % startTime)
sys.stdout.write("Stop listening at %s\n" % endTime)

securityOpts = {

	"serviceTimeLimit":1,
	"allowExecution":1,
	"restrictedExecution":0,
	"startTime":startTime,
	"endTime":endTime

	}

backUpActimdrc()

modifyActimRC(mapPath("~/.actimdrc"), None, securityOpts)

startActimdBg()

executeObj = popen2.Popen3(command, 1)

gotMessage = 0
mailTime = 0

# Setup the connection to the imap server
imapConn = imaplib.IMAP4(imapServer, port)
imapConn.login(user, pswd)
imapConn.select()

# Loop until we found a message or timesup
while (time() < later):
	
	# Use imap function to check out the receiving time of the
	# mail messsge
	try:
		result = imapConn.search(None, '(UNSEEN SUBJECT "%s")' % mailSubject)
	except imaplib.IMAP4.error, reason:
	    result = ('NOT OK', 'Search fail: %s\n' % reason)
	except imaplib.IMAP4.abort, reason:
	   	result = ('NOT OK', 'Search fail and abort: %s\n' % reason)
	except:
	   	result = ('NOT OK', 'Unexpected error\n')

	sys.stdout.write("%s, %s\n" % (result[0], result[1]))

	if result[0] == 'OK' and len(result[1][0]):
		gotMessage = 1
		break

receivedTime = None
# If found the message, then get the received time from the message
if gotMessage:
	numList = result[1][0]
	numList = string.split(numList)
	result, content = imapConn.fetch(numList[0],
									 "(BODY.PEEK[HEADER.FIELDS (DATE)])")
	if result == 'OK':
		receivedTime = content
		sys.stdout.write(">>>> Actim message received time: %s <<<<<<<\n" %
						 receivedTime)

		# The receivedTime is in a form of
		# [('38 (BODY[HEADER.FIELDS (DATE)] {41}', 'Date: .....'), ')']
		# ['...', ('38 (BODY[HEADER.FIELDS (DATE)] {41}', 'Date: .....'), ')']
		if type(receivedTime[0]) == types.TupleType:
			receivedTime = receivedTime[0][1]
		elif type(receivedTime[1]) == types.TupleType:
			receivedTime = receivedTime[1][1]
		else:
			pass

		sys.stdout.write(">>>> Extract received time is %s <<<<<<\n" % receivedTime)
		# strips the beginning "Date: "
		if receivedTime[:5] == "Date:":
			receivedTime = receivedTime[5:]

		mailTime = rfc822.parsedate(receivedTime)
	   	mailTime = mktime(mailTime)
		
	else:
		sys.stdout.write("%s, %s\n" % (result, content))


imapConn.close()
imapConn.logout()

# Now wait until the client returns
stdoutMsg = executeObj.fromchild.readlines()
stderrMsg = executeObj.childerr.readlines()
rtCode    = os.WEXITSTATUS(executeObj.wait())

if not gotMessage:
	numFailed = numFailed + 1
	sys.stdout.writelines(stdoutMsg)
	sys.stdout.writelines(stderrMsg)
else:
	# Check whether receive time in email is within the now and
	# later
	if mailTime >= later and mailTime <= endTime:
		sys.stdout.write(">>>>>> Warning: this test needs message arrived before the serviceTime <<<<<<\n")
		sys.stdout.write(">>>>>> Warning: You may need to perform run ntp <<<<<<\n")

	# Check the execution result of date command is within the
	# defined service time frame.

	sys.stdout.writelines(">>>> Fail: Mail receiving time is not within startTime and endTime <<<<<\n")
	sys.stdout.writelines(">>>> Mail receiving time: %s <<<<<\n" %
						  asctime(localtime(mailTime)))
	sys.stdout.writelines(">>>> startTime receiving time: %s <<<<<\n" %
						  asctime(localtime(later)))
	sys.stdout.writelines(">>>> endTime receiving time: %s <<<<<\n" %
						  asctime(localtime(end)))

	# Parse the XML result
	output = getEventExecuteResultOutput(None, None, stdoutMsg)
	if output != None:
		sys.stdout.writelines(">>>>> Output: %s <<<<<<\n" % output)
		
		# So far so good. Make sure the xmlOutput from the
		# actim command-line result is within the time frame
		outputSecs = float(output)
		if outputSecs >= later and outputSecs <= end:
			numPassed = numPassed + 1
		else:
			sys.stdout.write(">>>>>> Fail: returned time is outside the time frame <<<<<<\n")
			sys.stdout.write(">>>>>> Fail: returned time is %s <<<<<<\n" %
							 ctime(outputSecs))
			
			numFailed = numFailed + 1
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
	else:
		numFailed = numFailed + 1
		sys.stdout.write(">>>>>> Failed: No output element found <<<<<\n")
   		sys.stdout.writelines(stdoutMsg)
	   	sys.stdout.writelines(stderrMsg)
		
restoreActimdrc()
restoreActimrc()
stopServer()

report('test_serviceTime', numPassed, numFailed, None)
report('test_serviceTime', numPassed, numFailed)

