from MDC import *
from testTools import *
from ParseResult import *
import os, socket, re

numFailed = 0
numPassed = 0

# A dictionary of all the configuration in
# correct value format
clientConfigList = { 
					 'port': 17870,
					 'server':socket.gethostname(),
					 'requestRequest':1,
					 'resultAction':0,
					 'emailAddress':"abc@def.hij",
					 'emailPassword':"abc",
					 'emailAccount': "abcdefg",
					 'logStatus': 1,
					 'targetPath':mapPath("/tmp"),
					 'eKey':"avoid hanging",
					 'xmlOutput': 1
				   }


# 4 elements in a typle.
# first one is the option going to be tested
# second one is the value going to be teste
# third one is the expected return code
# forth one is the expected return message

# exit code 2 means trying to convert a value
# of an option to integer from string type. But
# the string value doesn't contain numeric character

# exit code 3 means the option has an empty value
testConfigs = [ ('port', "", 2, ""),
				('port', "abcd", 2, ""),
				('port', "17870", 0, ""),
				('server', None, 3, ""),
				('server', '', 3, ""),
				('requestResult', None, 2, ""),
				('requestResult', "abc", 2, ""),
				('requestResult', "0", 0, ""),
				('requestResult', "", 2, ""),
				('resultAction', "", 2, ""),
				('resultAction', "0", 0, ""),
				('resultAction', "abc", 2, ""),
				('emailAddress', None, 3, ""),
				('emailAddress', '', 3, ""),
				('emailAccount', '', 3, ""),
				('emailAccount', None, 3, ""),
				('emailPassword', None, 3, ""),
				('emailPassword', '', 3, ""),
				('logStatus', None, 2, ""),
				('logStatus', "error", 2, ""),
				('targetPath', None, 3, ""),
				('targetPath', "", 3, ""),
				('xmlOutput', "", 2, ""),
				('xmlOutput', "one", 2, ""),
				('xmlOutput', 1, 0, "")
			  ]

for config in testConfigs:

	option = config[0]
	value  = config[1]
	eRtn   = config[2]
	eRtnMsg= config[3]

	# update to options list
	testConfigList = clientConfigList.copy()
	testConfigList[option] = value
	
	sys.stdout.write("######### Set option: " + option +
					 " to value: " + `value` + ' ########\n')	

	# Backup the ~/,actimdrc
	backUpActimrc()

	# Create ~/.actimdrc from a dictionary
	createActimRCFile(testConfigList)
	
	if eRtn == 0:
		startActimdBg()
		stdoutMsg, stderrMsg, rtCode = mdc_popen("%s -C QUEUE_INFO ALL" %
												 actimCmd())

		if mdc_isLose():
			rtCode, dict, error = parseQueueInfoXMLResult(None, None, stdoutMsg)

		if rtCode == 0:
			sys.stdout.write(">>>>>> Server response seems fine <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Passed <<<<<<<<\n")
			numPassed = numPassed + 1
		else:
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
			if not mdc_isLose():
				os.system("cat ~/.actimdrc")
			sys.stdout.write(">>>>>> Fail: Server doesn't response <<<<<<<<<\n")
			numFailed = numFailed + 1

		stopServer()
	else:

		stdoutMsg, stderrMsg, rtnCode = mdc_popen("%s -C QUEUE_INFO ALL" %
												  actimCmd())
		if mdc_isLose():		
			rtnCode = getErrorExitCode(None, None, stdoutMsg)
		
		if rtnCode != eRtn:
			sys.stdout.write(">>>>>> Failed: return code doesn't match expected <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Expected return code " + `eRtn` + ", actual " +
							 `rtnCode` + " <<<<<<<\n")
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
			numFailed = numFailed + 1
		else:
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Passed <<<<<<<\n")
			numPassed = numPassed + 1

	restoreActimrc()

report('test_clientConfig', numPassed, numFailed, None)
report('test_clientConfig', numPassed, numFailed)

