from MDC import *
from testTools import *
import os
from ParseResult import *

numFailed = 0
numPassed = 0

# A dictionary of all the configuration in
# correct value format
daemonConfigList = { 
					 'port': 17870,
					 'SMTP_server':"smtp.foo.bar",
					 'SMTP_port': 8866,
					 'mailSubject': "ACTIM",
					 'IMAP4_server': "imap.foo.bar",
					 'IMAP4_port': 23456,
					 'numConnection': 5,
					 'logfile':'hello',
					 'logStatus': 0,
					 'logLevel': 99,
					 'saveRsltDir': '/tmp'
				   }


# 4 elements in a typle.
# first one is the option going to be tested
# second one is the value going to be tested
# third one is the expected return code
# forth one is the expected return message

testConfigs = [ ('port', "", 2, ""),
				('port', "abcd", 2, ""),
				('port', 17870, 0, ""),
				('SMTP_server', None, 3, ""),
				('SMTP_server', 'foo.bar.com', 0, ""),
				('SMTP_port', None, 2, ""),
				('SMTP_port', 12345, 0, ""),
				('SMTP_port', "abcd", 2, ""),
				('mailSubject', None, 3, ""),
				('mailSubject', 'Hello', 0, ""),
				('IMAP4_server', None, 3, ""),
				('IMAP4_server', 'foo.bar.com', 0, ""),
				('IMAP4_port', None, 2, ""),
				('IMAP4_port', 12345, 0, ""),
				('IMAP4_port', "abcd", 2, ""),
				('logStatus', None, 2, ""),
				('logStatus', "error", 2, ""),
				('saveRsltDir', None, 3, ""),
				('saveRsltDir', "error", 0, "")
				
			  ]

for config in testConfigs:

	option = config[0]
	value  = config[1]
	eRtn   = config[2]
	eRtnMsg= config[3]

	# update to options list
	testConfigList = daemonConfigList.copy()
	testConfigList[option] = value
	
	sys.stdout.write("######### Set option: " + option +
					 " to value: " + `value` + ' ########\n')	

	# Backup the ~/,actimdrc
	backUpActimdrc()

	# Create ~/.actimdrc from a dictionary
	createActimdRCFile(testConfigList)
	
	if eRtn == 0:
		startActimdBg()
		stdoutMsg, stderrMsg, rtCode = mdc_popen("%s -C QUEUE_INFO ALL" %
												 actimCmd())

		if mdc_isLose():
			rtCode, dict, error = parseQueueInfoXMLResult(None, None, stdoutMsg)

		if rtCode == 0:
			sys.stdout.write(">>>>>> Server response seems fine <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Passed <<<<<<<<\n")
			numPassed = numPassed + 1
		else:
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Fail: Server doesn't response <<<<<<<<<\n")
			numFailed = numFailed + 1

		stopServer()
	else:
		stdoutMsg, stderrMsg, rtnCode = mdc_popen(actimdCmd())

		if mdc_isLose():		
			rtnCode = getErrorExitCode(None, None, stdoutMsg)
		
		if rtnCode != eRtn:
			sys.stdout.write(">>>>>> Failed: return code doesn't match expected <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Expected return code " + `eRtn` + ", actual " +
							 `rtnCode` + " <<<<<<<\n")
			numFailed = numFailed + 1
		else:
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Passed <<<<<<<\n")
			numPassed = numPassed + 1

	restoreActimdrc()

report('test_daemonConfig', numPassed, numFailed, None)
report('test_daemonConfig', numPassed, numFailed)

