#!/usr/local/bin/python

import sys, os, time, os.path
from testTools import *
from LogLevel import *
from MDC import *

numFailed = 0
numPassed = 0

if mdc_isLose():
	os.exit(0)

os.system("rm -f /tmp/daemon.stderr.log")

# Enable the log, set log file to sys.stderr
# Set loglevel to the highest level. Run the daemon
# in background and also redirect to a file

# A list of tuples specifying the options,
# the file name where the log is expected to be
# 0 or 1 means whether to expect result or no result respectively
# Next parameter has 3 states, 0, 1, and 2.
# 0 - log redirects to a log file, 1 - redirects to stdout
# 2 - redirect to stderr
testConfigs = [
			 	("******** Set log output to stderr. Log switch on ********\n",
				 [('logfile', 'stderr'), ('logStatus', 1),
		   	 	  ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			 	 "/tmp/daemon.stderr.log", 1, 2),
			
			    ("******** Set log output to stderr. Log switch off *******\n",
				 [('logfile', 'stderr'), ('logStatus', 0),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stderr.log", 0, 2),
			
			    ("******** Set log output to stdout. Log switch on ********\n",
				 [('logfile', 'stdout'), ('logStatus', 1),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stdout.log", 1, 1),
			
			    ("******** Set log output to stdout. Log switch off *******\n",
				 [('logfile', 'stdout'), ('logStatus', 0),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stdout.log", 0, 1),
			
			    ("******** Set log output to a file. Log switch on ********\n",
				 [('logfile', '/tmp/daemon.stderr.log'),
				  ('logStatus', 1),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stderr.log", 1, 0),
			
			    ("******** Set log output to a file. Log switch off ******\n",
				 [('logfile', '/tmp/daemon.stderr.log'),
				  ('logStatus', 0),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stderr.log", 0, 0),
			
			    ("******** Set log output to a file. Log switch off ******\n",
				 [('logfile', '/tmp/daemon.stdout.log'),
			      ('logStatus', 1),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stdout.log", 1, 0),
			
				("******** Set log output to a file. Log switch off ******\n",
				 [('logfile', '/tmp/daemon.stdout.log'),
				  ('logStatus', 0),
		   	      ('logLevel', LogLevel.LOW_LEVEL_DEBUG)], 
			     "/tmp/daemon.stdout.log", 0, 0)
		]

for test in testConfigs:
	title = test[0]
	options = test[1]
	outputfile = test[2]
	expectContent = test[3]

	outputMode = test[4]

	sys.stdout.write(title)
	
	# Make sure to remove the content file first
	os.system("rm -fr " + outputfile)

	backUpActimdrc()

	if outputMode == 0: 
		cmd = "python ../../actimd &"
	elif outputMode == 1:
		cmd = "python ../../actimd 1> " + outputfile + " &"
	elif outputMode == 2:
		cmd = "python ../../actimd 2> " + outputfile + " &"

	sys.stdout.write("Server command is: " + cmd + '\n')
	
	changeActimdrc(options)
	
	startActimdBg(None, cmd)

	runCommand("python ../../actim -C QUEUE_INFO ALL")

	stopServer()
	time.sleep(3)

	restoreActimdrc()
	
	# Note that if outputMode is file mode and if
	# expectContent is off (logStatus), that means the file
	# shouldn't be created.
	if expectContent:

		if os.path.isfile(outputfile):
			# check the file size
			fileSize = os.stat(outputfile)[6]
   			sys.stdout.write("File " + outputfile + " exists, " +
	   						 "size: " + `fileSize` + '\n')
			if fileSize:
				sys.stdout.write(">>>>>> Passed <<<<<<<<\n")
				numPassed = numPassed + 1
				os.system("rm -f /tmp/daemon.*")
				continue
		else:
			sys.stdout.write("File " + outputfile + " doesn't exist\n")
			
		sys.stdout.write(">>>>>> Failed <<<<<<<<\n")
		numFailed = numFailed + 1
		
	else:
		if outputMode == 0:
			if os.path.isfile(outputfile):
	   			sys.stdout.write("File " + outputfile + " exists\n")
				sys.stdout.write(">>>>>> Failed <<<<<<<<<\n")
				numFailed = numFailed + 1
			else:
	   			sys.stdout.write("File " + outputfile + " doesn't exist\n")
				sys.stdout.write(">>>>>> Passed <<<<<<<<<\n")
				numPassed = numPassed + 1
		else:
			if os.path.isfile(outputfile):
				fileSize = os.stat(outputfile)[6]
	       		#sys.stdout.write("File " + outputfile + " exists, size: " + `fileSize` + '\n')
				sys.stdout.write(">>>>>> Passed <<<<<<<<\n")
				numPassed = numPassed + 1
			else:
				sys.stdout.write("File " + outputfile + " doesn't exist\n")
				sys.stdout.write(">>>>>> Failed <<<<<<<<\n")
				numFailed = numFailed + 1

	os.system("rm -f " + outputfile)
		
report('test_daemonLog', numPassed, numFailed, None)
report('test_daemonLog', numPassed, numFailed)

