from MDC import *
from testTools import *
import os
from ParseResult import *

numFailed = 0
numPassed = 0

# A dictionary of all the configuration in
# correct value format
daemonConfigList = { 
					 'port': 17870,
					 'SMTP_server':"foo.bar.com",
					 'IMAP4_server':"foo.bar.com",
					 'mailSubject':"ACTIM"
				   }

# Multiple configuration in a list and then
# each configuration is represented in a tuple
# The tuple contains 3 item, first - section,
# second - option name, third - option value
# the penultimate item is the test title
# the last item in the list is the expected
# exit code
testConfigs = [
				[ ("SECURITY", 'serviceTimeLimit', "1"),
				  ("SECURITY", 'startTime', "ab:cd:ef"),
				  ("SECURITY", 'endTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set startTime to alphabet *****\n" +
				   "******* Test parsing on startTime value *****\n"),
				  3 ],
				
				[ ("SECURITY", 'serviceTimeLimit', "0"),
				  ("SECURITY", 'startTime', "ab:cd:ef"),
				  ("SECURITY", 'endTime', "00:00:00"),
				  ("******* Set serviceTimeLimit off ***********\n" +
				   "******* Set startTime to alphabet *****\n" +
				   "******* Test parsing on startTime value *****\n"),
				  0 ],
				
				[ ("SECURITY", 'serviceTimeLimit', 1),
				  ("SECURITY", 'startTime', "12:00"),
				  ("SECURITY", 'endTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set startTime to not full format *****\n" +
				   "******* Test parsing on startTime value *****\n"),
				  3 ],
				
				[ ("SECURITY", 'serviceTimeLimit', 1),
				  ("SECURITY", 'startTime', "99:12:13"),
				  ("SECURITY", 'endTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set startTime to over value limit *****\n" +
				   "******* Test parsing on startTime value *****\n"),
				  3 ],
				
				[ ("SECURITY", 'serviceTimeLimit', "1"),
				  ("SECURITY", 'endTime', "ab:cd:ef"),
				  ("SECURITY", 'startTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set endTime to alphabet *****\n" +
				   "******* Test parsing on endTime value *****\n"),
				  3 ],
				
				[ ("SECURITY", 'serviceTimeLimit', "0"),
				  ("SECURITY", 'endTime', "ab:cd:ef"),
				  ("SECURITY", 'startTime', "00:00:00"),
				  ("******* Set serviceTimeLimit off ***********\n" +
				   "******* Set endTime to alphabet *****\n" +
				   "******* Test parsing on endTime value *****\n"),
				  0 ],
				
				[ ("SECURITY", 'serviceTimeLimit', 1),
				  ("SECURITY", 'endTime', "12:00"),
				  ("SECURITY", 'startTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set endTime to not full format *****\n" +
				   "******* Test parsing on endTime value *****\n"),
				  3 ],
				
				[ ("SECURITY", 'serviceTimeLimit', 1),
				  ("SECURITY", 'startTime', "99:12:13"),
				  ("SECURITY", 'endTime', "00:00:00"),
				  ("******* Set serviceTimeLimit on ***********\n" +
				   "******* Set endTime to over value limit *****\n" +
				   "******* Test parsing on endTime value *****\n"),
				  3 ]
			  ]

for config in testConfigs:

	eRtn   = config[-1]
	title  = config[-2]

	# update to options list
	testConfigList = daemonConfigList.copy()
	securityList = {}

	sys.stdout.write(title)

	for optionPair in config[:-2]:
		if optionPair[0] == "SECURITY":
			securityList[optionPair[1]] = optionPair[2]
		elif optionPair[0] == None:
			testConfigList[optionPair[1]] = optionPair[2]

		sys.stdout.write("######### Set option: %s to value %s in section %s\n"
						 % (optionPair[1], optionPair[2], optionPair[0]))
		
	# Backup the ~/,actimdrc
	backUpActimdrc()

	modifyActimRC(mapPath("~/.actimdrc"), testConfigList, securityList)

	if eRtn == 0:
		startActimdBg()
		stdoutMsg, stderrMsg, rtCode = mdc_popen("%s -C QUEUE_INFO ALL" %
												 actimCmd())
		if rtCode == 0:
			sys.stdout.write(">>>>>> Server response seems fine <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Passed <<<<<<<<\n")
			numPassed = numPassed + 1
		else:
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Fail: Server doesn't response <<<<<<<<<\n")
			numFailed = numFailed + 1

		stopServer()
	else:
		stdoutMsg, stderrMsg, rtnCode = mdc_popen(actimdCmd())

		if mdc_isLose():		
			rtnCode = getErrorExitCode(None, None, stdoutMsg)
		
		if rtnCode != eRtn:
			sys.stdout.writelines(stdoutMsg)
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Failed: return code doesn't match expected <<<<<<<<<\n")
			sys.stdout.write(">>>>>> Expected return code " + `eRtn` + ", actual " +
							 `rtnCode` + " <<<<<<<\n")
			numFailed = numFailed + 1
		else:
			sys.stdout.writelines(stderrMsg)
			sys.stdout.write(">>>>>> Passed <<<<<<<\n")
			numPassed = numPassed + 1

	restoreActimdrc()

report('test_daemonMultipleConfig', numPassed, numFailed, None)
report('test_daemonMultipleConfig', numPassed, numFailed)
